import 'package:yona/yona.dart';
import 'dart:math' as math;

var host = NetAddr.fromString("yonafediverse.com:7777");

void main() async {
  var user1 = await Crypto.genSK();
  var user2 = await Crypto.genSK();
  var packet = Packet(Crypto.derivePK(user1), Crypto.derivePK(user2),
      await Crypto.randBytes(100), Packet.zeroID);
  var zeroPacket = Packet(Crypto.derivePK(user1), List.filled(Packet.pkSize, 0), [],
      Packet.zeroID);
  List<Packet> packets = [zeroPacket];
  var timings = [];
  for (var i = 0; i < 10; i++) {
    var startSend = DateTime.now();
    await Longlog.push(host, packet, user1, Duration(seconds: 2));
    var endSend = DateTime.now();
    var startRecv = DateTime.now();
    packets = await Longlog.pull(host, zeroPacket, 10,Duration(seconds: 2));
    var endRecv = DateTime.now();
    assert(packets.length == math.min(i+1,10));
    for (var pack in packets) {
      assert(Utils.equal(pack.data, packet.data));
      assert(Utils.equal(pack.from, packet.from));
      assert(Utils.equal(pack.to, packet.to));
    }
    timings.add({
      "send": endSend.difference(startSend).inMicroseconds,
      "recv": endRecv.difference(startRecv).inMicroseconds,
    });
    print(i);
  }
  print("send timeline ${timings.map((e) => e["send"])}");
  print("send total ${timings.map((e) => e["send"]).reduce((value, element) => value+element)}");
  print("send each ${timings.map((e) => e["send"]).reduce((value, element) => value+element)/timings.length/1000000}");
  print("recv timeline ${timings.map((e) => e["recv"])}");
  print("recv total ${timings.map((e) => e["recv"]).reduce((value, element) => value+element)}");
  print("recv each ${timings.map((e) => e["recv"]).reduce((value, element) => value+element)/timings.length/1000000}");

}
