import 'package:yona/src/msg.dart';
import 'package:yona/src/node.dart';
import 'net_addr.dart';
import 'cache.dart';
import 'package:messagepack/messagepack.dart' as msgpack;
class Client {
  final Cache cache;
  final List<int> secretKey;
  final NetAddr longlog;
  Client(this.cache,this.secretKey,this.longlog);
  Future<void> _setLast(List<int> last){
    return cache.set(longlog.toString(), last);
  }
  Future<List<int>> _getLast()async{
    return  (await cache.get(longlog.toString())) ?? [];
  }
  Future<void> publish(String name){
    return Node.publish(secretKey, longlog, name);
  }
  Future<String> lookupName(List<int> publicKey,Duration timeout ) async{
    var info = await Node.lookup(publicKey, timeout);
    return info.name;
  }
  Future<void> send(List<int> toPublicKey,DateTime sentAt,int kind,List<int> payload,Duration timeout) async{
    var data = msgpack.Packer();
    {
      data.packInt(sentAt.millisecondsSinceEpoch);
      data.packInt(kind);
      data.packBinary(payload);
    }
    return Node.send(timeout, secretKey, Send(toPublicKey, Msg(sentAt,kind,payload)));
  }

  bool _isReceiving = false;
  Future<List<Recv>> receive(int limit ) async{
    if (_isReceiving) return [];
    _isReceiving = true;
    try {
     var packets = await _receive(limit);
     if (packets.isNotEmpty){
       await _setLast(packets.last.id);
     }
     return packets;
    }finally{
      _isReceiving = false;
    }
  }
  Future <List<Recv>> _receive(int limit) async{
    var packets = await Node.receive(secretKey, longlog,limit ,await _getLast(),Duration(seconds: 3));
    return packets;
  }
}
