import 'dart:async';
import 'dart:convert';
import 'packet.dart';
import 'crypto.dart';
import 'package:http/http.dart' as http;
import 'net_addr.dart';
import 'package:messagepack/messagepack.dart' as msgpack;
class ServiceError implements Exception{
  final int status;
  final String msg;
  ServiceError(this.status,this.msg);
}

class Longlog {
  static Future<List<Packet>> pull(NetAddr host ,Packet last, int limit,Duration timeout) async{
    var cli = http.Client();
    var timer = Timer(timeout,(){
      try {
        cli.close();
      } catch (_) {}
    });
    try {
      var resp = await cli.put(
          host.toUri(
              scheme: "http", queryParameters: {"limit": limit.toString()}),
          body: last.encode());
      switch (resp.statusCode) {
        case 200:
          break;
        case 204:
          return [];
        default:
          throw ServiceError(resp.statusCode, resp.body);
      }
      var pack = msgpack.Unpacker(resp.bodyBytes);
      var len = pack.unpackListLength();
      List<Packet> result = [];
      for (var i = 0; i < len; i++) {
        result.add(Packet.decode(pack.unpackBinary()));
      }
      return result;
    }finally{
      try {
        cli.close();
      } catch (_) {}
      if (timer.isActive) timer.cancel();
    }
  }
  static Future<void> push(
      NetAddr addr, Packet packet, List<int> sk, Duration timeout) async {
    var cli = http.Client();
    var timer = Timer(timeout, () {
      try {
        cli.close();
      } catch (_) {}
    });
    var data = packet.encode();
    try {
      var resp = await cli.post(addr.toUri(scheme: "http"),
          headers: {
            "Content-Signature": base64UrlEncode(await Crypto.sign(sk, data))
          },
          body: data);
      if (resp.statusCode != 204) {
        throw Exception(resp.body.isEmpty ? resp.statusCode : resp.body);
      }
    } finally {
      try {
        cli.close();
        if (timer.isActive) {
          timer.cancel();
        }
      } catch (_) {}
    }
  }
}
